@@ -59,6 +59,7 @@ static GObjectClass *parent_class = NULL;
 
 enum {
 	ZOOM_REGION_MANAGED_PROP,
+	ZOOM_REGION_POLL_MOUSE_PROP,
 	ZOOM_REGION_SMOOTHSCROLL_PROP,
 	ZOOM_REGION_INVERT_PROP,
 	ZOOM_REGION_SMOOTHING_PROP,
@@ -83,6 +84,7 @@ enum {
 gchar* prop_names[ZOOM_REGION_EXIT_MAGNIFIER + 1] = 
 {
     "MANAGED",
+    "POLLMOUSE"
     "SMOOTHSCROLL",
     "INVERT",
     "SMOOTHING",
@@ -1397,7 +1399,8 @@ zoom_region_update_pointer (ZoomRegion *zoom_region, gboolean draw_cursor)
 #ifdef ZOOM_REGION_DEBUG
 	g_assert (zoom_region->alive);
 #endif
-	if (!zoom_region->priv || !zoom_region->priv->parent)
+	if (!zoom_region->priv || !zoom_region->priv->parent 
+	    || !zoom_region->poll_mouse)
 	      return FALSE; 
 
 	magnifier = zoom_region->priv->parent;
@@ -2225,6 +2228,9 @@ zoom_region_get_property (BonoboPropertyBag *bag,
 	case ZOOM_REGION_MANAGED_PROP:
 		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->is_managed);
 		break;
+	case ZOOM_REGION_POLL_MOUSE_PROP:
+		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->poll_mouse);
+		break;
 	case ZOOM_REGION_INVERT_PROP:
 		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->invert);
 		break;
@@ -2308,6 +2314,25 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 	case ZOOM_REGION_MANAGED_PROP:
 		zoom_region->is_managed = BONOBO_ARG_GET_BOOLEAN (arg);
 		break;
+	case ZOOM_REGION_POLL_MOUSE_PROP:
+		zoom_region->poll_mouse = BONOBO_ARG_GET_BOOLEAN (arg);
+		if (zoom_region->poll_mouse)
+		{
+		    g_message ("Adding polling timer");
+		    zoom_region->priv->update_pointer_id =
+			g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
+					    200,
+					    zoom_region_update_pointer_timeout,
+					    zoom_region,
+					    NULL);
+		}
+		else if (zoom_region->priv->update_pointer_id)
+		{
+		    g_message ("Removing polling timer");
+		    g_source_remove (zoom_region->priv->update_pointer_id);
+		    zoom_region->priv->update_pointer_id = 0;
+		}
+		break;
 	case ZOOM_REGION_INVERT_PROP:
 		zoom_region->invert = BONOBO_ARG_GET_BOOLEAN (arg);
 		zoom_region_update_current (zoom_region);
@@ -2500,6 +2525,47 @@ zoom_region_pan_test (gpointer data)
 	return FALSE;
 }
 
+static void
+impl_zoom_region_set_pointer_pos (PortableServer_Servant servant,
+				  const CORBA_long mouse_x,
+				  const CORBA_long mouse_y,
+				  CORBA_Environment *ev)
+{
+	ZoomRegion *zoom_region =
+		ZOOM_REGION (bonobo_object_from_servant (servant));
+	GdkRectangle paint_area, *clip = NULL;
+
+#ifdef ZOOM_REGION_DEBUG
+	g_assert (zoom_region->alive);
+#endif
+	DBG (fprintf (stderr, "Set Pointer: \t%ld,%ld\n", 
+		      (long) mouse_x, (long) mouse_y));
+
+	fprintf (stderr, "Set Pointer: \t%ld,%ld\n", 
+		      (long) mouse_x, (long) mouse_y);
+
+	zoom_region_set_cursor_pos (zoom_region, (int) mouse_x, (int) mouse_y);
+
+	if (GTK_IS_WIDGET (zoom_region->priv->w) && 
+	    GDK_IS_DRAWABLE (zoom_region->priv->w->window))
+	{
+	    gdk_drawable_get_size (
+		GDK_DRAWABLE (
+		    zoom_region->priv->w->window),
+		&paint_area.width, &paint_area.height);
+	    paint_area.x = 0;
+	    paint_area.y = 0;
+	    clip = &paint_area;
+	    paint_area = zoom_region_clip_to_source (
+		zoom_region, paint_area);
+	}
+	/* 
+	 * if we update the cursor now, it causes flicker if the client 
+	 * subsequently calls setROI, so we wait for a redraw.
+	 * Perhaps we should cue a redraw on idle instead?
+	 */
+}
+
 static void
 impl_zoom_region_set_contrast (PortableServer_Servant servant,
 			       const CORBA_float R,
@@ -2822,6 +2888,7 @@ zoom_region_class_init (ZoomRegionClass *klass)
 	epv->getMagFactor = impl_zoom_region_get_mag_factor;
 	epv->getProperties = impl_zoom_region_get_properties;
 	epv->setROI = impl_zoom_region_set_roi;
+	epv->setPointerPos = impl_zoom_region_set_pointer_pos;
 	epv->markDirty = impl_zoom_region_mark_dirty;
 	epv->getROI = impl_zoom_region_get_roi;
 	epv->moveResize = impl_zoom_region_move_resize;
@@ -2861,6 +2928,19 @@ zoom_region_properties_init (ZoomRegion *zoom_region)
 				 Bonobo_PROPERTY_READABLE |
 				 Bonobo_PROPERTY_WRITEABLE);
 
+	bonobo_arg_release (def);
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, TRUE);
+	
+	bonobo_property_bag_add (zoom_region->properties,
+				 "poll-mouse",
+				 ZOOM_REGION_POLL_MOUSE_PROP,
+				 BONOBO_ARG_BOOLEAN,
+				 NULL,
+				 "If false, zoom region does not poll for pointer location, but is (exclusively) given it by the client",
+				 Bonobo_PROPERTY_READABLE |
+				 Bonobo_PROPERTY_WRITEABLE);
+
 	bonobo_arg_release (def);
 	def = bonobo_arg_new (BONOBO_ARG_SHORT);
 	BONOBO_ARG_SET_SHORT (def, GNOME_Magnifier_ZoomRegion_SCROLL_FASTEST);
@@ -3147,6 +3227,7 @@ zoom_region_init (ZoomRegion *zoom_region)
 	zoom_region->x_align_policy = GNOME_Magnifier_ZoomRegion_ALIGN_CENTER;
 	zoom_region->y_align_policy = GNOME_Magnifier_ZoomRegion_ALIGN_CENTER;
 	zoom_region->coalesce_func = _coalesce_update_rects;
+	zoom_region->poll_mouse = TRUE; 
 	zoom_region->priv = g_malloc (sizeof (ZoomRegionPrivate));
 	zoom_region_private_init (zoom_region->priv);
 	bonobo_object_add_interface (BONOBO_OBJECT (zoom_region),
